# LinkExchange.Django - Django integration with LinkExchange library
# Copyright (C) 2009, 2011 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

from django.db import transaction

from linkexchange.db_drivers import BaseMultiHashDriver
from linkexchange_django.models import DBHash, DBHashItem

class DjangoMultiHashDriver(BaseMultiHashDriver):
    """
    Django ORM multihash driver.
    """
    def __init__(self, dbname):
        self.dbname = dbname

    def load(self, hashkey):
        try:
            return DBHash.objects.get(dbname=self.dbname, key=hashkey)
        except DBHash.DoesNotExist:
            raise KeyError(hashkey)

    def get_mtime(self, hashkey):
        try:
            return DBHash.objects.get(dbname=self.dbname, key=hashkey).mtime
        except DBHash.DoesNotExist:
            raise KeyError(hashkey)

    def _run_in_transaction(self, func, args=[], kwargs={}):
        ok = False
        sid = transaction.savepoint()
        try:
            result = func(*args, **kwargs)
            ok = True
        finally:
            if ok:
                transaction.savepoint_commit(sid)
            else:
                transaction.savepoint_rollback(sid)
        return result

    def _get_hash(self, hashkey):
        try:
            return DBHash.objects.get(dbname=self.dbname, key=hashkey)
        except DBHash.DoesNotExist:
            pass
        h = DBHash(dbname=self.dbname, key=hashkey)
        h.save()
        return h

    def save(self, hashkey, newhash, blocking=True):
        def _save():
            h = self._get_hash(hashkey)
            h.set_items(newhash)
            h.save()
            return True
        return self._run_in_transaction(_save)

    def modify(self, hashkey, otherhash, blocking=True):
        def _modify():
            h = self._get_hash(hashkey)
            h.update_items(otherhash)
            h.save()
            return True
        return self._run_in_transaction(_modify)

    def delete(self, hashkey, keys, blocking=True):
        def _delete():
            h = self._get_hash(hashkey)
            h.delete_items(keys)
            h.save()
            return True
        return self._run_in_transaction(_delete)



import sys
import unittest

def additional_tests():
    if sys.version_info < (2, 7) and 'nosetests' not in sys.argv:
        print >> sys.stderr, (
                "ERROR: not compatible Python version: %s" % sys.version.split()[0])
        print >> sys.stderr, (
                "This tests uses class level fixtures that is not supported\n"
                "by unittest on Python prior version 2.7.\n"
                "To run tests on Python < 2.7 use the nose extension\n"
                "<http://pypi.python.org/pypi/nose/>\n"
                "\n"
                "  $ easy_install nose\n"
                "  $ python setup.py nosetests\n"
                )
        sys.exit(1)
    return unittest.TestSuite()



# LinkExchange.Django - Django integration with LinkExchange library
# Copyright (C) 2009, 2011 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

from linkexchange_django import support
from linkexchange.utils import rearrange_blocks, parse_rearrange_map

def linkexchange(request):
    if support.platform is None:
        return {}

    page_request = support.convert_request(request)
    result = {}

    if support.formatters:
        result['linkexchange_blocks'] = support.platform.get_blocks(
                page_request, support.formatters)
        try:
            rearrange_map = parse_rearrange_map(support.options['rearrange_map'])
        except KeyError:
            pass
        except ValueError:
            log.warning("Unable to parse rearrange_map")
        else:
            result['linkexchange_blocks'] = rearrange_blocks(page_request,
                    result['linkexchange_blocks'], rearrange_map)

    if support.options.get('use_raw_links', False):
        result['linkexchange_links'] = support.platform.get_raw_links(
                page_request)

    return result



import unittest
import sys
import os
import tempfile
import shutil
import urllib2
import time

from linkexchange.clients.sape import SapeTestServer
from linkexchange.clients.sape import SapeArticlesIndexTestServer
from linkexchange.clients.sape import SapeArticlesArticleTestServer
from linkexchange.clients.sape import SapeArticlesTemplateTestServer
from linkexchange.tests import WebAppProcess

class MySapeTestServer(SapeTestServer):
    data = {
        '/': [
            '<a href="http://example1.com">example text 1</a>',
            '<a href="http://example2.com">example text 2</a>',
            ],
        '/path/1': [
            '<a href="http://example1.com">example text 1</a>',
            '<a href="http://example2.com">example text 2</a>',
            '<a href="http://example3.com">example text 3</a>',
            '<a href="http://example4.com">example text 4</a>',
            ],
        '__sape_new_url__': '<!--12345-->',
        '__sape_delimiter__': '. ',
        }

class MySapeContextTestServer(SapeTestServer):
    data = {
        '/': [
            'Test <a href="http://example1.com">context</a> link 1.',
            ],
        '/path/1': [
            'Test <a href="http://example2.com">context</a> link 2.',
            ],
        '__sape_new_url__': '<!--12345-->',
        }

class MySapeArticlesIndexTestServer(SapeArticlesIndexTestServer):
    data = {
            'announcements': {
                '/': [
                    '<a href="/articles/1">example announcement link</a>'],
                },
            'articles': {
                '/articles/1': {
                    'id': '1',
                    'date_updated': '0',
                    'template_id': '1',
                    },
                },
            'images': {},
            'templates': {
                '1': {
                    'lifetime': '3600',
                    },
                },
            'template_fields': [
                'title', 'keywords', 'header', 'body',
                ],
            'template_required_fields': [
                'title', 'keywords', 'header', 'body',
                ],
            'ext_links_allowed': [],
            'checkCode': '<!--12345-->',
            'announcements_delimiter': '|',
            }

class MySapeArticlesArticleTestServer(SapeArticlesArticleTestServer):
    data = {
            'date_updated': '0',
            'title': 'The article title',
            'keywords': 'The keywords',
            'header': 'The article header',
            'body': '<p>The article <a href="http://example.com">body</a>.</p>',
            }

class MySapeArticlesTemplateTestServer(SapeArticlesTemplateTestServer):
    raw_data = """
    <html>
      <head>
        <title>{title}</title>
        <meta name="keywords" content="{keywords}"/>
      </head>
    <body>
      <h1>{header}</h1>
      {body}
    </body>
    </html>
    """

class LinkExchangeDjangoTest(unittest.TestCase):
    settings_py = """
from os.path import dirname, join

DEBUG = True
TEMPLATE_DEBUG = DEBUG
ADMINS = ()
MANAGERS = ADMINS
DATABASE_ENGINE = 'sqlite3'
DATABASE_NAME = join(dirname(__file__), 'site.db')
TIME_ZONE = 'Europe/Kiev'
LANGUAGE_CODE = 'ru-ru'
SITE_ID = 1
USE_I18N = True
MEDIA_ROOT = join(dirname(__file__), 'static')
MEDIA_URL = '/static/'

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
)

ROOT_URLCONF = 'testprj.urls'

TEMPLATE_DIRS = (
    join(dirname(__file__), 'templates'),
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'linkexchange_django',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.core.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'django.core.context_processors.request',
    'linkexchange_django.context_processors.linkexchange',
    )

LINKEXCHANGE_CONFIG = join(dirname(__file__), 'linkexchange.cfg')
"""
    linkexchange_cfg = """
[client-1]
type = sape
user = user12345
db_driver.type = shelve
db_driver.filename = %%(basedir)s/sape-XXX.db
server-1 = %(sape_server)s

[client-2]
type = sape_context
user = user12345
db_driver.type = shelve
db_driver.filename = %%(basedir)s/sape_context-XXX.db
server-1 = %(sape_context_server)s

[client-3]
type = sape_articles
user = user12345
index_db_driver.type = shelve
index_db_driver.filename = %%(basedir)s/sape-articles-index-XXX.db
article_db_driver.type = shelve
article_db_driver.filename = %%(basedir)s/sape-articles-article-XXX.db
image_db_driver.type = shelve
image_db_driver.filename = %%(basedir)s/sape-articles-image-XXX.db
template_db_driver.type = shelve
template_db_driver.filename = %%(basedir)s/sape-articles-template-XXX.db
index_server-1 = %(sape_art_index_server)s
article_server-1 = %(sape_art_article_server)s

[formatter-1]
type = list
count = 2
class_ = linx
prefix = "-&nbsp;"
suffix = ". "

[formatter-2]
type = inline
count = 2
class_ = linx
delimiter = " | "
prolog = "| "
epilog = " |"

[formatter-3]
type = list
count = none
class_ = linx
prefix = "+&nbsp;"
suffix = ". "
"""
    urls_py = """
from django.conf.urls.defaults import *
urlpatterns = patterns('',
    (r'^articles/\d+$', 'linkexchange_django.views.handle_request'),
    (r'^.*', 'django.views.generic.simple.direct_to_template',
        {'template': 'test.html'}),
)
"""

    test_html = """
{% load linkexchange_tags %}
<html>
<body>

{% linkexchange_filter request %}
<p>
Test context link 1.
</p>
{% endlinkexchange_filter %}

{% with "Test context link 2." as text %}
<p>
{{ text|linkexchange_filter:request }}
</p>
{% endwith %}

<p>Location 1: {{ linkexchange_blocks.0|safe }}</p>
<p>Location 2: {{ linkexchange_blocks.1|safe }}</p>
<p>Location 3: {{ linkexchange_blocks.2|safe }}</p>

</body>
<html>
"""

    def setUpClass(cls):
        cls.prevdir = os.getcwd()
        cls.topdir = tempfile.mkdtemp()
        os.chdir(cls.topdir)

        # search for django-admin
        django_admin = None
        dirs = filter(lambda x: x, os.getenv('PATH', '').split(os.pathsep))
        dirs.append(os.path.dirname(sys.executable))
        dirs.append(os.path.join(os.path.dirname(sys.executable), 'Scripts'))
        files = ['django-admin.py', 'django-admin-script.py',
                'django-admin', 'django-admin.exe']
        for d in dirs:
            for f in files:
                p = os.path.join(d, f)
                if os.path.exists(p):
                    django_admin = p
                    break
            if django_admin:
                break
        assert django_admin
        if django_admin.endswith('.py'):
            django_admin_args = [sys.executable, django_admin]
        else:
            django_admin_args = [django_admin]

        # call django-admin startproject
        os.spawnv(os.P_WAIT, django_admin_args[0],
                django_admin_args + ['startproject', 'testprj'])

        cls.projectdir = os.path.join(cls.topdir, 'testprj')
        os.chdir(cls.projectdir)
        os.mkdir('templates')

        cls.sape_server = MySapeTestServer()
        cls.sape_context_server = MySapeContextTestServer()
        cls.sape_art_template_server = MySapeArticlesTemplateTestServer()
        template_urls = [('1', cls.sape_art_template_server.url)]
        cls.sape_art_index_server = MySapeArticlesIndexTestServer(
                template_urls=template_urls)
        cls.sape_art_article_server = MySapeArticlesArticleTestServer()

        open('settings.py', 'w').write(cls.settings_py)
        q = lambda s: s.replace('%', '%%')
        open('linkexchange.cfg', 'w').write(cls.linkexchange_cfg % {
            'sape_server': q(cls.sape_server.url),
            'sape_context_server': q(cls.sape_context_server.url),
            'sape_art_index_server': q(cls.sape_art_index_server.url),
            'sape_art_article_server': q(cls.sape_art_article_server.url),
            })
        open('urls.py', 'w').write(cls.urls_py)
        open(os.path.join('templates', 'test.html'), 'w').write(cls.test_html)

        cls.manage_py = [sys.executable, 'manage.py']
        cls.host, cls.port = ('127.0.0.1', 8000)
        
    setUpClass = classmethod(setUpClass)

    def tearDownClass(cls):
        del cls.sape_server
        del cls.sape_context_server
        del cls.sape_art_index_server
        del cls.sape_art_article_server
        del cls.sape_art_template_server
        os.chdir(cls.prevdir)
        shutil.rmtree(cls.topdir)
    tearDownClass = classmethod(tearDownClass)

    def test_test(self):
        retcode = os.spawnv(os.P_WAIT, self.manage_py[0],
                self.manage_py + ['test', 'linkexchange_django'])
        self.assertEqual(retcode, 0)

    def test_validate(self):
        retcode = os.spawnv(os.P_WAIT, self.manage_py[0],
                self.manage_py + ['validate'])
        self.assertEqual(retcode, 0)

    def test_lxrefresh(self):
        retcode = os.spawnv(os.P_WAIT, self.manage_py[0],
                self.manage_py + ['syncdb', '--noinput'])
        self.assertEqual(retcode, 0)
        retcode = os.spawnv(os.P_WAIT, self.manage_py[0],
                self.manage_py + ['lxrefresh',
                    '--host=%s:%d' % (self.host, self.port)])
        self.assertEqual(retcode, 0)

    def test_run(self):
        os.spawnv(os.P_WAIT, self.manage_py[0],
                self.manage_py + ['syncdb', '--noinput'])

        success = False
        django_server = WebAppProcess(self.manage_py +
                ['runserver', str(self.port)], (self.host, self.port))
        try:
            baseurl = 'http://%s:%d' % (self.host, self.port)

            body = urllib2.urlopen(baseurl + '/').read()
            self.assertEqual('<html>' in body, True)

            self.assertEqual(
                    '<a href="http://example1.com">example text 1</a>' in body, True)
            self.assertEqual(
                    '<a href="http://example2.com">example text 2</a>' in body, True)
            self.assertEqual(
                    'Test <a href="http://example1.com">context</a> link 1.' in body, True)
            self.assertEqual(
                    'Test <a href="http://example2.com">context</a> link 2.' in body, False)
            self.assertEqual(
                    '<a href="/articles/1">example announcement link</a>' in body, True)

            body = urllib2.urlopen(baseurl + '/path/1').read()
            self.assertEqual(
                    '<a href="http://example3.com">example text 3</a>' in body, True)
            self.assertEqual(
                    '<a href="http://example4.com">example text 4</a>' in body, True)
            self.assertEqual(
                    'Test <a href="http://example1.com">context</a> link 1.' in body, False)
            self.assertEqual(
                    'Test <a href="http://example2.com">context</a> link 2.' in body, True)

            body = urllib2.urlopen(baseurl + '/articles/1').read()
            self.assertEqual(
                    '<title>The article title</title>' in body, True)
            self.assertEqual(
                    '<h1>The article header</h1>' in body, True)
            self.assertEqual(
                    '<a href="http://example.com">' in body, True)

            success = True
        finally:
            if not success:
                time.sleep(3)
            sys.stdout.flush()
            django_server.terminate()
            django_server.wait()



# LinkExchange.Django - Django integration with LinkExchange library
# Copyright (C) 2009, 2011 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

import datetime
try:
    import cPickle as pickle
except ImportError:
    import pickle

from django.db import models

class DBHash(models.Model):
    """
    DBHash represents first level item in multihash, e.g. hash with its own
    items, that may be accessed by key as item of multihash object.
    """
    dbname = models.CharField(
            help_text="Name of multihash database.",
            max_length=100, db_index=True)
    key = models.CharField(
            help_text="Hash key.",
            max_length=100, db_index=True)
    mtime = models.DateTimeField(
            help_text="Hash modification time.",
            auto_now_add=True)

    class Meta:
        unique_together = (
                ('dbname', 'key'),
                )

    def __unicode__(self):
        return "%s:%s" % (self.dbname, self.key)

    def __len__(self):
        return self.items.all().count()

    def __getitem__(self, key):
        try:
            return pickle.loads(str(self.items.get(key=key).value))
        except DBHashItem.DoesNotExist:
            raise KeyError(key)

    def __setitem__(self, key):
        raise RuntimeError("Hash is read only")

    def __iter__(self):
        for item in self.items.all():
            yield item.key

    def __contains__(self, item):
        return self.items.filter(key=item).count() > 0

    def get(self, key, default=None):
        try:
            return self[key]
        except KeyError:
            return default

    def clear_items(self):
        self.items.all().delete()
        self.mtime = datetime.datetime.now()

    def update_items(self, newhash):
        if isinstance(newhash, dict):
            newhash = newhash.items()
        for key, value in newhash:
            try:
                record = self.items.get(key=key)
                record.value = pickle.dumps(value)
            except DBHashItem.DoesNotExist:
                record = self.items.create(key=key,
                        value=pickle.dumps(value))
            record.save()
        self.mtime = datetime.datetime.now()

    def set_items(self, newhash):
        if isinstance(newhash, dict):
            newhash = newhash.items()
        newhash = [(key, pickle.dumps(value))
                for key, value in newhash]
        newhash.sort()
        oldhash = [(x.key, x.value) for x in self.items.all()]
        oldhash.sort()

        i = 0
        while i < max(len(newhash), len(oldhash)):
            if i < len(newhash):
                nkey, nval = newhash[i]
            else:
                nkey = nval = None
                kdiff = 1
            if i < len(oldhash):
                okey, oval = oldhash[i]
            else:
                okey = oval = None
                kdiff = -1
            if nkey and okey:
                kdiff = cmp(nkey, okey)
            if kdiff == 0:
                if nval != oval:
                    item = self.items.get(key=okey)
                    item.value = nval
                    item.save()
                i += 1
            elif kdiff < 0:
                item = self.items.create(key=nkey, value=nval)
                item.save()
                oldhash.insert(i, (nkey, nval))
                i += 1
            elif kdiff > 0:
                self.items.filter(key=okey).delete()
                del oldhash[i]
        self.mtime = datetime.datetime.now()

    def delete_items(self, keys):
        for k in keys:
            self.items.filter(key=k).delete()
        self.mtime = datetime.datetime.now()

class DBHashItem(models.Model):
    hash = models.ForeignKey(DBHash,
            db_index=True, related_name='items')
    key = models.CharField(max_length=1000, db_index=True)
    value = models.TextField()

    class Meta:
        unique_together = (
                ('hash', 'key'),
                )

    def __unicode__(self):
        return self.key



# LinkExchange.Django - Django integration with LinkExchange library
# Copyright (C) 2009, 2011 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

from django.http import Http404, HttpResponse

from linkexchange_django import support

def handle_request(request):
    if support.platform is None:
        raise Http404

    page_request = support.convert_request(request)
    page_response = support.platform.handle_request(page_request)

    if page_response.status == 404 and not page_response.body:
        raise Http404

    headers = page_response.headers.copy()
    response = HttpResponse(page_response.body,
            content_type=headers.pop('Content-Type', 'text/html'),
            status=page_response.status)
    for k, v in headers.items():
        response[k] = v

    return response



# LinkExchange.Django - Django integration with LinkExchange library
# Copyright (C) 2009, 2011 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

import sys
import logging
from optparse import make_option

from django.core.management.base import BaseCommand, CommandError
from django.contrib.sites.models import Site

from linkexchange.clients import PageRequest
from linkexchange.utils import configure_logger
from linkexchange_django import support

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
            make_option('--host', '-d', dest = 'host',
                help = 'Request host.'),
            make_option('--uri', '-i', dest = 'uri',
                help = 'Request URI.'),
            )
    help = "Force to refresh LinkExchange clients databases."

    def handle(self, *args, **options):
        logger = logging.getLogger('linkexchange')
        log_level = min(50, max(10,
            10 + 20 * (2 - int(options.get('verbosity', 1)))))
        logger.setLevel(log_level)
        configure_logger()

        uri = options.get('uri', '/')
        host = options.get('host', support.options.get('host', None))

        if support.platform is None:
            sys.stderr.write(self.style.ERROR("No platform defined\n"))
            sys.exit(1)

        if host is None:
            if Site._meta.installed:
                current_site = Site.objects.get_current()
                if current_site is not None:
                    host = current_site.domain
            if host is None:
                sys.stderr.write(self.style.ERROR("No host configured\n"))
                sys.exit(1)

        page_request = PageRequest(host=host, uri=uri)
        support.platform.refresh_db(page_request)




# LinkExchange.Django - Django integration with LinkExchange library
# Copyright (C) 2011 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

import unittest

from linkexchange.tests import MultiHashDriverTestMixin

from linkexchange_django.models import DBHash
from linkexchange_django.db_drivers import DjangoMultiHashDriver

class DBHashTest(unittest.TestCase):
    def setUp(self):
        self.hash = DBHash.objects.create(dbname='testdb', key='testkey')
        self.hash.save()

    def tearDown(self):
        self.hash.items.all().delete()
        self.hash.delete()

    def test_clear_items(self):
        self.hash.set_items([('k1', 'v1'), ('k2', 'v2')])
        self.hash.save()
        self.assertEqual(len(self.hash), 2)
        self.hash.clear_items()
        self.hash.save()
        self.assertEqual(len(self.hash), 0)

    def test_update_items(self):
        self.hash.set_items([('k1', 'v1'), ('k2', 'v2'), ('k3', 'v3')])
        self.hash.save()
        self.assertEqual(len(self.hash), 3)

        self.hash.update_items([('k3', 'v3x'), ('k4', 'v4')])
        self.hash.save()
        self.assertEqual(len(self.hash), 4)
        self.assertEqual(self.hash['k1'], 'v1')
        self.assertEqual(self.hash['k2'], 'v2')
        self.assertEqual(self.hash['k3'], 'v3x')
        self.assertEqual(self.hash['k4'], 'v4')

    def test_set_items(self):
        self.hash.set_items([('k1', 'v1'), ('k2', 'v2'), ('k3', 'v3')])
        self.hash.save()
        self.assertEqual(len(self.hash), 3)
        self.assertEqual(self.hash['k1'], 'v1')
        self.assertEqual(self.hash['k2'], 'v2')
        self.assertEqual(self.hash['k3'], 'v3')

        self.hash.set_items([('k1', 'v1'), ('k3', 'v3x'), ('k4', 'v4')])
        self.hash.save()
        self.assertEqual(len(self.hash), 3)
        self.assertEqual(self.hash['k1'], 'v1')
        self.assertEqual(self.hash['k3'], 'v3x')
        self.assertEqual(self.hash['k4'], 'v4')

    def test_delete_items(self):
        self.hash.set_items([('k1', 'v1'), ('k2', 'v2'), ('k3', 'v3')])
        self.hash.save()
        self.assertEqual(len(self.hash), 3)
        self.hash.delete_items(['k2', 'k3'])
        self.assertEqual(len(self.hash), 1)

class DjangoMultiHashDriverTest(MultiHashDriverTestMixin,
        unittest.TestCase):

    with_blocking = False

    def setUp(self):
        self.db = DjangoMultiHashDriver('testdb')

    def tearDown(self):
        for h in DBHash.objects.filter(dbname=self.db.dbname):
            h.items.all().delete()
            h.delete()
        del self.db



# LinkExchange.Django - Django integration with LinkExchange library
# Copyright (C) 2009, 2011 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

from django import template
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe
from django.utils.encoding import force_unicode

from linkexchange_django import support

register = template.Library()

def linkexchange_filter(value, request, autoescape=None):
    """
    Django template filter to support linkexchange content filtering.  The
    argument is request object, to access this object add
    'django.core.context_processors.request' to the TEMPLATE_CONTEXT_PROCESSORS
    in your settings.py.

    Usage example:

        {% load linkexchange_tags %}
        {{ page.html|safe|linkexchange_filter:request }}
    """
    if autoescape:
        esc = conditional_escape
    else:
        esc = lambda x: x

    value = esc(force_unicode(value))

    if support.platform is not None:
        value = support.platform.content_filter(
                support.convert_request(request), value)

    return mark_safe(value)

linkexchange_filter.needs_autoescape = True
register.filter('linkexchange_filter', linkexchange_filter)

class LinkExchangeFilterNode(template.Node):
    def __init__(self, request, nodelist):
        self.request = request
        self.nodelist = nodelist

    def render(self, context):
        request = self.request.resolve(context)
        content = self.nodelist.render(context)
        if support.platform is not None:
            content = support.platform.content_filter(
                    support.convert_request(request), content)
        return content

def linkexchange_filter_tag(parser, token):
    """
    Django template tag to support linkexchange content filtering.  The
    argument is request object, to access this object add
    'django.core.context_processors.request' to the TEMPLATE_CONTEXT_PROCESSORS
    in your settings.py.

    Usage example:

        {% load linkexchange_tags %}
        <html>
        <body>
            {% linkexchange_filter request %}
            Page content.
            {% endlinkexchange_filter %}
        </body>
        </html>
    """
    try:
        tag_name, request = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError, (
                "%r tag requires a single argument" %
                token.contents.split()[0])
    request = parser.compile_filter(request)
    nodelist = parser.parse(('endlinkexchange_filter',))
    parser.delete_first_token()
    return LinkExchangeFilterNode(request, nodelist)

register.tag('linkexchange_filter', linkexchange_filter_tag)



from setuptools import setup, find_packages
setup(
    name="LinkExchange.Django",
    version="0.1",
    packages=find_packages(exclude=['tests']),
    author="Konstantin Korikov",
    author_email="lostclus@gmail.com",
    url="http://linkexchange.org.ua",
    download_url="http://linkexchange.org.ua/downloads",
    description="Django integration with LinkExchange library",
    long_description="""
    This package contains Django support code for LinkExchange library.
    """,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        ],
    license = "LGPL",
    install_requires=[
        'LinkExchange>=0.4dev',
        ],
    test_suite='tests',
    tests_require=[
        'phpserialize',
        'Django',
        ],
    entry_points="""
    [linkexchange.multihash_drivers]
    django = linkexchange_django.db_drivers:DjangoMultiHashDriver
    """,
)



# LinkExchange.Django - Django integration with LinkExchange library
# Copyright (C) 2009, 2011 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

import os
import os.path
import urllib
import logging

from django.conf import settings
from django.contrib.sites.models import Site, RequestSite
from django.utils.encoding import iri_to_uri

from linkexchange.config import file_config, ConfigError
from linkexchange.utils import normalize_uri
from linkexchange.platform import Platform
from linkexchange.clients import PageRequest

log = logging.getLogger('linkexchange.django')

platform = None
formatters = None
options = None

def configure():
    def check_mod_dir(mod):
        if isinstance(mod, basestring):
            try:
                mod = __import__(mod, {}, {}, [''])
            except ImportError:
                return None
        fn = os.path.join(os.path.dirname(mod.__file__), 'linkexchange.cfg')
        if not os.path.exists(fn):
            return None
        return fn

    try:
        real_settings = __import__(os.environ['DJANGO_SETTINGS_MODULE'],
                {}, {}, [''])
    except KeyError:
        real_settings = None

    try:
        cfgfile = settings.LINKEXCHANGE_CONFIG
    except AttributeError:
        cfgfile = None
    if not cfgfile and real_settings:
        cfgfile = check_mod_dir(real_settings)
    if not cfgfile:
        cfgfile = check_mod_dir(settings.ROOT_URLCONF)

    if cfgfile:
        defaults = {}
        if isinstance(cfgfile, basestring):
            defaults['basedir'] = os.path.abspath(
                    os.path.dirname(cfgfile))
        elif real_settings:
            defaults['basedir'] = os.path.abspath(
                    os.path.dirname(real_settings.__file__))
        else:
            defaults['basedir'] = os.getcwd()
        try:
            if not file_config(globals(), cfgfile, defaults=defaults):
                log.error("Unable to read configuration file: %s", cfgfile)
        except ConfigError, e:
            log.error("Configuration error: %s", str(e))

    global platform
    global formatters
    global options

    try:
        platform = Platform(settings.LINKEXCHANGE_CLIENTS)
    except AttributeError:
        pass
    try:
        formatters = settings.LINKEXCHANGE_FORMATTERS
    except AttributeError:
        pass
    if options is None:
        options = {}
    try:
        options.update(settings.LINKEXCHANGE_OPTIONS)
    except AttributeError:
        pass

    if platform is None:
        log.warning("LinkExchange is not configured")

def convert_request(request):
    if Site._meta.installed:
        current_site = Site.objects.get_current()
    else:
        current_site = RequestSite(request)

    path = request.path
    if type(path) == unicode:
        path = path.encode('utf-8')
    query_string = iri_to_uri(request.environ.get('QUERY_STRING', ''))
    request_uri = urllib.quote(path) + (query_string and
            ('?' + query_string) or '')

    request = PageRequest(
            host=options.get('host', current_site.domain),
            uri=normalize_uri(request_uri),
            cookies=request.COOKIES,
            remote_addr=request.META.get('REMOTE_ADDR', None),
            meta=request.META)
    return request

configure()
